      SUBROUTINE CLOUDPHOT(SZA,QLIQ,XPHFM,FCC,PHOTOFACT,
     +                     IMAX,NIV)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Chemistry
C   Purpose   : Compute modification of photolyses according to cloudiness
C               according to [Chang et al., 1987]. 
C   Called by : 
C
CCC
C
C  Original : V.-H. Peuch, 09/2000
C
C  Modified : H. Teyssedre, (optimisation), 01/2005
C
CCC
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C SZA     : SOLAR ZENITHAL ANGLE
C QLIQ    : LIQUID WATER CONTENT          (g water / g air)
C XPHFM   : PRESSURE AT HALF MODEL LEVELS (Pa)
C FCC     : CLOUD COVER AT EACH LEVEL
C PHOTOFACT : MODIFICATION FOR CLEAR-SKY PHOTOLYSES
C
      INTEGER :: IMAX,NIV
      REAL (KIND=8) , DIMENSION (IMAX,NIV) :: QLIQ, XPHFM, FCC
      REAL (KIND=8) , DIMENSION (IMAX)     :: SZA
      REAL (KIND=8) , DIMENSION (IMAX,NIV) :: PHOTOFACT
      REAL (KIND=8) :: COSXPIS3
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~	
C
C FCCUP   : AVERAGE CLOUD COVER ABOVE EACH LEVEL
C FCCDN   : AVERAGE CLOUD COVER BELOW EACH LEVEL
C ZZTAU   : TOTAL CLOUD EXTINCTION OPTICAL DEPTH
C ZZTAUUP : CLOUD EXTINCTION OPTICAL DEPTH ABOVE EACH LEVEL
C ZZTAUDN : CLOUD EXTINCTION OPTICAL DPETH BELOW EACH LEVEL
C TRUP    : TRANSMISSIVITY BETWEEN CURRENT LEVEL AND MODEL TOP
C TRDN    : TRANSMISSIVITY NETWEEN SURFACE AND CURRENT LEVEL
C
      INTEGER :: I,INIV
      REAL (KIND=8) :: ZFACT,XPI,CSZA,XFUP,XFDN,XFIN
      REAL (KIND=8) , ALLOCATABLE  :: ZZTAU(:)
      REAL (KIND=8) , ALLOCATABLE  :: FCCUP(:,:),FCCDN (:,:)
      REAL (KIND=8) , ALLOCATABLE  :: ZZTAUUP(:,:),ZZTAUDN(:,:)
      REAL (KIND=8) , ALLOCATABLE  :: TRUP(:,:),TRDN(:,:)
C
      ALLOCATE (ZZTAU(IMAX))
      ALLOCATE (FCCUP(IMAX,NIV))
      ALLOCATE (FCCDN(IMAX,NIV))
      ALLOCATE (ZZTAUUP(IMAX,NIV))
      ALLOCATE (ZZTAUDN(IMAX,NIV))
      ALLOCATE (TRUP(IMAX,NIV))
      ALLOCATE (TRDN(IMAX,NIV))
C
CCC
C
C ZFACT=(A+B/R)/g [Slingo, 1989; Hack et al., 1993]
C
C A : 280 cm2 g-1
C B : 13000 10-6m cm2 g-1
C R : 10 10-6m
C g : 981 cm s-2
C
      ZFACT=(280.+13000./10.)/981.
      XPI=2.*ASIN(1.)
C
C COMPUTE CLOUD COVER ABOVE AND BELOW EACH LEVEL
C
      DO I=1,IMAX
        FCCUP(I,1)=1.
      ENDDO
      DO iniv=2,NIV
        DO I=1,IMAX
          FCCUP(I,iniv)=FCCUP(I,iniv-1)*(1.-FCC(I,iniv))
        ENDDO
      ENDDO
C
      DO I=1,IMAX
        FCCDN(I,NIV)=1.
      ENDDO
      DO iniv=NIV-1,1,-1
        DO I=1,IMAX
          FCCDN(I,iniv)=FCCDN(I,iniv+1)*(1.-FCC(I,iniv))
        ENDDO
      ENDDO
C
      DO I=1,IMAX*NIV
        FCCUP(I,1)=1.-FCCUP(I,1)
        FCCDN(I,1)=1.-FCCDN(I,1)
      ENDDO
C
C COMPUTE TOTAL CLOUD OPTICAL DEPTH
C
      DO I=1,IMAX
        ZZTAU(I)=QLIQ(I,1)*XPHFM(I,1)*(FCC(I,1)**1.5)
      ENDDO
      DO iniv=2,NIV
        DO I=1,IMAX
          ZZTAU(I)=ZZTAU(I) +
     +          QLIQ(I,iniv)*(FCC(I,iniv)**1.5)*
     +          (XPHFM(I,iniv)-XPHFM(I,iniv-1))
        ENDDO
      ENDDO
      DO I=1,IMAX
        ZZTAU(I)=ZZTAU(I)*ZFACT
      ENDDO
C
C COMPUTE CLOUD OPTICAL DEPTH AND TRANSMITTIVITY
C ABOVE AND BELOW CURRENT LEVEL
C
      DO I=1,IMAX
        ZZTAUUP(I,1) = 0.
        ZZTAUDN(I,1) = ZZTAU(I)
        TRUP(I,1)    = 1.
        TRDN(I,1)    = (5. - EXP(ZZTAU(I)))
     +                /(4. + 0.42*ZZTAU(I))
      ENDDO
      DO iniv=2,NIV
        DO I=1,IMAX
          ZZTAUUP(I,iniv)=ZZTAUUP(I,iniv-1)+
     +          QLIQ(I,iniv)*(FCC(I,iniv)**1.5)*
     +          (XPHFM(I,iniv)-XPHFM(I,iniv-1))*ZFACT
          ZZTAUDN(I,iniv)=ZZTAU(I)-ZZTAUUP(I,iniv)
          TRUP(I,iniv) = (5. - EXP(ZZTAUUP(I,iniv)))
     +                  /(4. + 0.42*ZZTAUUP(I,iniv))
          TRDN(I,iniv) = (5. - EXP(ZZTAUDN(I,iniv)))
     +                  /(4. + 0.42*ZZTAUDN(I,iniv))
        ENDDO
      ENDDO
C
C COMPUTE MODIFICATION FACTOR
C
      COSXPIS3=0.5
      DO iniv=1,NIV
        DO I=1,IMAX
          CSZA=MAX(COS(SZA(I)*XPI/180.),COSXPIS3)
          XFUP=(1.-TRDN(I,iniv))*CSZA*FCCDN(I,iniv)
          XFDN=(1.6*CSZA*TRUP(I,iniv)-1.)*FCCUP(I,iniv)
          XFIN=(1.4*CSZA-1)*FCC(I,iniv)
          PHOTOFACT(I,iniv)=MAX(1.e-10,1.+XFUP+XFIN+XFDN)
        ENDDO
      ENDDO
C
      DEALLOCATE (ZZTAU)
      DEALLOCATE (FCCUP)
      DEALLOCATE (FCCDN)
      DEALLOCATE (ZZTAUUP)
      DEALLOCATE (ZZTAUDN)
      DEALLOCATE (TRUP)
      DEALLOCATE (TRDN)
C
      RETURN
      END
