SUBROUTINE KFBCONVMA(LON,LAT,NIV,NTRA,IDEB,IFIN,JDEB,JFIN,XPHFM,SURFM,QQQM,LDEB,AM,AK)
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!   DATE : 15/05/2005
!
!   Type       : Convection and Diffusion (Bechtold/Louis)
!   Purpose    : Compute diffusion matrix for one vertical column
!                given vertical diffusion coefficients
!   Called by  :
!
!
!
!  Original : B. Josse & V.-H. Peuch, 04/2000
!
!  Modified : B. Josse & V.-H. Peuch, 07/2002
!	      H. Le Berre, 09/2006 , modification interface d'appel , passage de variables
!	      en argument pour eviter des tests et instructions redondantes sur les 4 domaines
!	      H. Le Berre, 07/2007, Amelioration de la vectorisation, traitement 3D
!
!
!
! INPUT 
!
!	AM:	AIRMASS (KG) PER M**2 IN EACH LAYER
!       EU:	ENTRAINMENT IN UPDRAFT IN EACH LAYER   (KG M**-2 S**-1)
!	DU:	DETRAINMENT IN UPDRAFT IN EACH LAYER   (KG M**-2 S**-1)
!	ED:	ENTRAINMENT IN DOWNDRAFT IN EACH LAYER (KG M**-2 S**-1)
!	DD:	DETRAINMENT IN DOWNDRAFT IN EACH LAYER (KG M**-2 S**-1)
!	AK:	DIFFUSION COEFFICIENT ON LAYER BOUNDARIES
!		TIMES DENSITY/DISTANCE BETWEEN ADJACENT LAYER CENTERS
!		(KG M**-2 S**-1)
!	DTCONV:	TIMESTEP (S)
!
!       OUTPUT :
!
!	G:	CONVECTION MATRIX
!
! NB. : VERTICAL COORDINATE IS ASSUMED TO GO FROM TOP TO BOTTOM,
!
!
USE EXPE_MOD
USE CSTES_MOD
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
INTEGER :: LON,LAT,NIV,NTRA,LDEB,IDEB,IFIN,JDEB,JFIN
REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPHFM
REAL (KIND=8) , DIMENSION (LON,LAT) :: SURFM
REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
REAL (KIND=8) , DIMENSION (LON,LAT,LDEB:NIV) :: AM
REAL (KIND=8) , DIMENSION (LON,LAT,LDEB:NIV) :: AK
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
INTEGER :: I,J,IJ,K,L,JV
REAL (KIND=8) :: COEFG,COEFD
REAL (KIND=8) , ALLOCATABLE :: SM(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: XPHFM_R(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: G(:,:,:,:),F(:,:,:,:)
REAL (KIND=8) , ALLOCATABLE :: U1(:,:,:),U2(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: V1(:,:,:),V2(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: RMXCOL(:,:,:),RMYCOL(:,:,:)
!
!
! 0. INITILIZATIONS
!
ALLOCATE (SM(LON,LAT,NIV))
ALLOCATE (XPHFM_R(LON,LAT,0:NIV))
ALLOCATE (G(LON,LAT,LDEB:NIV,LDEB:NIV))
ALLOCATE (F(LON,LAT,LDEB-1:NIV,LDEB:NIV))
!
!
DO IJ=1,LON*LAT
  XPHFM_R(IJ,1,0)=200.
ENDDO
!    
DO L=1,NIV
  DO IJ=1,LON*LAT
    XPHFM_R(IJ,1,L)=XPHFM(IJ,1,L)      
  ENDDO
ENDDO    
!
DO L=LDEB,NIV
  DO K=LDEB-1,NIV
    DO IJ=1,LON*LAT
      F(IJ,1,K,L)=0.
    ENDDO
  ENDDO  
!
  DO K=LDEB,NIV
    DO IJ=1,LON*LAT
      G(IJ,1,K,L)=0.     
    ENDDO
  ENDDO
ENDDO
!
!
DO L=1,NIV
  DO IJ=1,LON*LAT
    SM(IJ,1,L)=ABS(XPHFM_R(IJ,1,L)-XPHFM_R(IJ,1,L-1))*SURFM(IJ,1)/GG
  ENDDO
ENDDO
!
DEALLOCATE (XPHFM_R)
!
! 1. DIFFUSION TERMS
!
DO K=LDEB,NIV-1
  DO IJ=1,LON*LAT
    F(IJ,1,K,K)  =F(IJ,1,K,K)   - AK(IJ,1,K)/AM(IJ,1,K)
    F(IJ,1,K,K+1)=F(IJ,1,K,K+1) + AK(IJ,1,K)/AM(IJ,1,K+1)
  ENDDO
ENDDO
!
! 2. COMPUTE FORWARD MATRIX
!
DO L=LDEB,NIV
  DO K=LDEB,NIV
    DO IJ=1,LON*LAT
      G(IJ,1,K,L)=DTCONV*(F(IJ,1,K-1,L) - F(IJ,1,K,L))
    ENDDO
  ENDDO
ENDDO
!
DEALLOCATE (F)
!
DO K=LDEB,NIV
  DO IJ=1,LON*LAT  
    G(IJ,1,K,K)=G(IJ,1,K,K) + 1.0
  ENDDO
ENDDO
!
! 3. INVERT TRIDIAGONAL MATRIX
!
ALLOCATE (U1(LON,LAT,LDEB:NIV))
ALLOCATE (V1(LON,LAT,LDEB:NIV))
!
!$omp parallel sections default(none) private(IJ) &
!$omp shared(LON,LAT,LDEB,NIV,U1,V1,G)
!$omp section
DO IJ=1,LON*LAT
    U1(IJ,1,LDEB)=G(IJ,1,LDEB,LDEB)
ENDDO  
DO L=LDEB+1,NIV
  DO IJ=1,LON*LAT
    U1(IJ,1,L)=G(IJ,1,L,L)-G(IJ,1,L-1,L)*G(IJ,1,L,L-1)/U1(IJ,1,L-1)
  ENDDO
ENDDO
!$omp section
DO IJ=1,LON*LAT
    V1(IJ,1,NIV)=G(IJ,1,NIV,NIV)
ENDDO
DO L=NIV-1,LDEB,-1
  DO IJ=1,LON*LAT
    V1(IJ,1,L)=G(IJ,1,L,L)-G(IJ,1,L+1,L)*G(IJ,1,L,L+1)/V1(IJ,1,L+1)
  ENDDO
ENDDO
!$omp end parallel sections
!
! INITIALIZE VECTOR Y
!
!
!$omp parallel default(none) private(I,J,IJ,JV,K,COEFG,COEFD,RMXCOL,RMYCOL,U2,V2) &
!$omp shared(LON,LAT,NTRA,IDEB,IFIN,JDEB,JFIN,LDEB,NIV,SM,U1,V1,G,QQQM)
!
ALLOCATE (RMXCOL(LON,LAT,LDEB:NIV))
ALLOCATE (RMYCOL(LON,LAT,LDEB:NIV))
ALLOCATE (U2(LON,LAT,LDEB:NIV))
ALLOCATE (V2(LON,LAT,LDEB:NIV))
!
!$omp do
DO JV=1,NTRA
!
  DO K=LDEB,NIV
    DO IJ=1,LON*LAT
      RMYCOL(IJ,1,K)=QQQM(IJ,1,K,JV)*SM(IJ,1,K)
    ENDDO  
  ENDDO
!
  DO IJ=1,LON*LAT
    U2(IJ,1,LDEB)=RMYCOL(IJ,1,LDEB)
  ENDDO 
!   
  DO L=LDEB+1,NIV
    DO IJ=1,LON*LAT
      U2(IJ,1,L)=RMYCOL(IJ,1,L)-G(IJ,1,L,L-1)*U2(IJ,1,L-1)/U1(IJ,1,L-1)
    ENDDO
  ENDDO
!
  DO IJ=1,LON*LAT
    V2(IJ,1,NIV)=RMYCOL(IJ,1,NIV)
  ENDDO
!      
  DO L=NIV-1,LDEB,-1
    DO IJ=1,LON*LAT  
      V2(IJ,1,L)=RMYCOL(IJ,1,L)-G(IJ,1,L,L+1)*V2(IJ,1,L+1)/V1(IJ,1,L+1)
    ENDDO  
  ENDDO
!   
  DO L=LDEB,NIV
    DO IJ=1,LON*LAT
      COEFG=U1(IJ,1,L)+V1(IJ,1,L)-G(IJ,1,L,L)
      COEFD=U2(IJ,1,L)+V2(IJ,1,L)-RMYCOL(IJ,1,L)
      RMXCOL(IJ,1,L)=COEFD/COEFG
    ENDDO
  ENDDO
!
  DO L=LDEB,NIV
    DO J=JDEB,JFIN
      DO I=IDEB,IFIN
        QQQM(I,J,L,JV)=RMXCOL(I,J,L)/SM(I,J,L)
      ENDDO 
    ENDDO
  ENDDO
!
ENDDO
!$omp enddo
!
DEALLOCATE (RMXCOL)
DEALLOCATE (RMYCOL)
DEALLOCATE (U2)
DEALLOCATE (V2)
!
!$omp end parallel
!
DEALLOCATE (SM)
DEALLOCATE (G)
DEALLOCATE (U1)
DEALLOCATE (V1)
!
!
RETURN
END SUBROUTINE KFBCONVMA
