      SUBROUTINE KFBLOUIS(LON,LAT,NIV,LDEB,PZ,PT,PU,PV,PK)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Bechtold/Louis) 
C   Purpose   : Compute vertical diffusion coefficients based
C               upon vertical stability of atmosphere     
C   Called by : 
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified : H. Le Berre, 07/2007, Amelioration de la vectorisation, traitement 3D
C
CCC
C
C     INPUT
C     (all variables are defined on level boundaries)
C		PZ	GEOPOTENTIAL HEIGHT 	(M)
C		PT	TEMPERATURE  		(K)
C		PU	U-WINDCOMPONENT 	(M/S)
C		PV	V-WINDCOMPONENT 	(M/S)
C
C     OUPUT
C     (variables are defined on level boundaries)
C		PK	VERTICAL DIFFUSION COEFFICIENT
C						(KG M**-2 S**-1)
C
C     Reference
C
C     LOUIS, J.-F., A PARAMETRIC MODEL OF VERTICAL EDDY FLUXES IN THE
C     ATMOSPHERE, BOUNDARY LAYER METEOROLOGY, 17, 187-202, 1979.
C
CCCCC
C
      USE CSTES_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LON,LAT,NIV,LDEB
      REAL (KIND=8) , DIMENSION (LON,LAT,0:NIV) :: PZ,PT,PU,PV
      REAL (KIND=8) , DIMENSION (LON,LAT,LDEB:NIV) :: PK
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~	
C
      INTEGER :: I,J,JL,NIVM1
      REAL (KIND=8) :: PPDV2MIN, PPB,PPC,PPD,PPALAMH,PPAK
      REAL (KIND=8) :: ZH,ZAL2,ZDZM,ZDV2,ZRI,ZFH
      PARAMETER (PPDV2MIN=0.01)
      PARAMETER(PPB=5.,PPC=5.,PPD=5.,PPALAMH=438.18,PPAK=0.4)
C 	MINIMAL VELOCITY DIFFERENCE SQUARED (M**2/S**2)
C
C
      NIVM1=NIV-1
      DO JL=LDEB,NIVM1
      DO J=1,LAT
      DO I=1,LON
C
C 1. HEIGHT ABOVE GROUND (GEOPOT. HEIGHT MIGHT BE NEGATIVE!)
C
        ZH=PZ(I,J,JL)-PZ(I,J,NIV)
C
C 2. MIXING LENGTH SQUARED
C
        ZAL2=(PPAK*ZH/(1.+PPAK*ZH/PPALAMH))**2
C
C 3. HEIGHT DIFFERENCE BETWEEN CENTERS OF ADJACENT LAYERS
C
        ZDZM=(PZ(I,J,JL-1)-PZ(I,J,JL+1))/2.
C
C 4. SQUARED VELOCITY DIFFERENCE BETWEEN ADJACENT LAYERS
C
        ZDV2=AMAX1(PPDV2MIN,((PU(I,J,JL-1)-PU(I,J,JL+1))**2 +
     +                       (PV(I,J,JL-1)-PV(I,J,JL+1))**2)/4.)
C
C 5. RICHARDSON NUMBER
C
        ZRI=GG*ZDZM*(CP*(PT(I,J,JL-1)-PT(I,J,JL+1))/2.+GG*ZDZM)/
     +         (CP*PT(I,J,JL)*ZDV2)
C
C 6. STABILITY FUNCTIONS
C
        IF(ZRI.LT.0.) THEN
          ZFH=1.0 - 3.0*PPB*ZRI/
     +          (1.0 + 3.0*PPB*PPC*ZAL2*
     +        SQRT( -ZRI/ZH*
     +              (((1.+ZDZM/ZH)**(1./3.)-1.)/ZDZM)**3))
        ELSE
          ZFH=1./(1.0 + 3.0*PPB*ZRI*SQRT(1.0 + PPD*ZRI))
        ENDIF
C
C 7. DIFFUSION COEFFICIENT
C
        PK(I,J,JL)=ZAL2*ZFH*SQRT(ZDV2)/ZDZM
C
      ENDDO
      ENDDO
      ENDDO
C
      PK(:,:,NIV)=0.
C
      RETURN
      END
