SUBROUTINE LESSIV_PREC                     &
           (IOPT,                          &
            LON,LAT,NIV,NTRA,SURF,         &
            LSTPRECFLX,SSTPRECFLX,         &
            XPHFM,XPAM,TGRIM,QLIQ,QICE,    &
            XKHENRY,TRANAM,QQQM)        
!
!
! Computes scavenging of species due to 
! precipitations. 
! Based on a modified Giorgi and Chameides scheme (1986)
! used by Liu et al (2000) 
! Rainout (or in-cloud scavenging), washout (or below-cloud
! scavenging) and reevaporation are computed.
!
!      Original : Beatrice Josse, september 2001
!      Modified : H. Le Berre Mars 2007
!                 Optimisation, vectorisation et parallelisation
!      
!      
USE CSTES_MOD
!
IMPLICIT NONE
!
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
!   scavenging option
!   if iopt=0, only washout is treated (below-cloud scavenging)
!   if iopt=1, rainout and washout are treated
! 
INTEGER :: IOPT
INTEGER :: LON,LAT,NIV,NTRA
REAL (KIND=8) , DIMENSION (LON,LAT) :: SURF
! Geometry of the model
REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: LSTPRECFLX
! 3D liquid stratiform precipitation flux
! (kg water/ m2/s) 
REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: SSTPRECFLX
! 3D solid stratiform precipitation flux
! (kg water/ m2/s) 
REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPHFM,XPAM
! Pressure (Pa) at level boundaries and centers
REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: TGRIM
! Temperature(K)
REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: QLIQ,QICE
!  Liquid and Ice water content (kg water/kg air)
REAL (KIND=8) , DIMENSION (NTRA,2) :: XKHENRY
! Henry's constant values for species
CHARACTER(LEN=15) , DIMENSION (NTRA) ::  TRANAM
!  Names of species
REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) ::  QQQM
! species mixing ratios (mol of species/mol of air)
!
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
INTEGER :: I,L,JC
INTEGER :: LTOP,LBOT
#ifdef MASSy
INTEGER :: iHNO3  , iN2O5  , iCLONO2, iCLNO2 , iBRONO2, iHNO4
INTEGER :: iHONO  , iONIT  , iPAN   , iTPAN  , iOLNN  , iOLND
INTEGER :: iNOx   , iNOy   , iOCLO  , iHCL   , iHOCL  , iCL2
INTEGER :: iCLOx  , iCLy   , iBRCL  , iHBR   , iHOBR  , iBROx
INTEGER :: iBRy
#endif
REAL (KIND=8) :: RHO,ZKHI_1
REAL (KIND=8) :: ZKHC,ZKHI,RKJC
REAL (KIND=8) :: RLAMBDA,DELTANJC,RKSEC,DENUM,RPRECMOY
REAL (KIND=8) :: RWC,RNJC
REAL (KIND=8) :: COEF1,COEF2,QQM1,QQM2,QQM
REAL (KIND=8) , PARAMETER :: XT0=298.
!   reference  temperature(K) for Henry's constants
REAL (KIND=8) , PARAMETER :: XT0_INV=1./XT0
REAL (KIND=8) , PARAMETER :: RKMIN=1.E-4
! minimum value (in s-1) of rk, frequency for
! conversion of cloud water to precipitation
REAL (KIND=8) , PARAMETER :: RKPRIM=1.E2
! (m-1) . Washout loss rate constant for aerosols
!  and HNO3
REAL (KIND=8) , PARAMETER :: DELTAT=1800.
!  (s) time step 
REAL (KIND=8) , PARAMETER :: ZHP=1.E-5
! [H+]
REAL (KIND=8) , PARAMETER :: RLPW=1.5E-6
REAL (KIND=8) , PARAMETER :: SEUIL=1.E-8
REAL (KIND=8) , PARAMETER :: EPSILON=1.E-14
REAL (KIND=8) , PARAMETER :: ZKHI_2=1./273.
REAL (KIND=8) , PARAMETER :: ZKHI_3=34./18./(34./29.)
!
REAL (KIND=8) , ALLOCATABLE :: RPREC(:,:,:),RPRECM1(:,:,:)
!1D Stratiform precipitation flux (m3 water/m2 surface/s) 
REAL (KIND=8) , ALLOCATABLE :: ZKH(:,:,:)
! Effective Henry's law constants (mol/L/atm)
REAL (KIND=8) , ALLOCATABLE :: ALPHA(:,:,:),BETA(:,:,:)
! Fraction of species in the liquid/cloud condensate
REAL (KIND=8) , ALLOCATABLE :: RIM(:)
! Retention efficiency of tracer during riming 
REAL (KIND=8) , ALLOCATABLE :: TRAC(:,:,:)
!  Moles of Tracer accumulated over the column
!  by scavenving
REAL (KIND=8) , ALLOCATABLE :: VALDEB(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: DELTAP(:,:,:),DELTAZ(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: RNAIR(:,:,:),Q(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: XT(:,:,:),XP(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: XSDELTAT(:,:,:),ZLWCC(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: ZLWCI(:,:,:),ZKS1(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: ZKS2(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: FRAC(:,:,:),RK(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: FRACSTOCK(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: XPHFMM1(:,:,:)
INTEGER       , ALLOCATABLE :: IIN(:,:,:)      
#ifdef MASSy
REAL (KIND=8) , ALLOCATABLE :: NOx  (:,:,:),HNO3  (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: N2O5 (:,:,:),HNO4  (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: CLONO2(:,:,:),ClNO2(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: BrONO2(:,:,:)
REAL (KIND=8) , ALLOCATABLE :: HONO (:,:,:),ONIT  (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: PAN  (:,:,:),OLNN (:,:,:)
#ifdef RACMOBUS
REAL (KIND=8) , ALLOCATABLE :: TPAN (:,:,:),OLND (:,:,:)
#endif
REAL (KIND=8) , ALLOCATABLE :: ClOx (:,:,:),OCLO  (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: HCl  (:,:,:),HOCl  (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: Cl2  (:,:,:),BrCl  (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: BrOx (:,:,:),HBr   (:,:,:)
REAL (KIND=8) , ALLOCATABLE :: HOBr (:,:,:)
#endif
!
!
ALLOCATE (RPREC(LON,LAT,0:NIV))
ALLOCATE (RPRECM1(LON,LAT,0:NIV))
ALLOCATE (RIM(NTRA))
ALLOCATE (TRAC(LON,LAT,NTRA))
ALLOCATE (VALDEB(LON,LAT,NIV))
ALLOCATE (DELTAP(LON,LAT,NIV))
ALLOCATE (DELTAZ(LON,LAT,NIV))
ALLOCATE (RNAIR(LON,LAT,NIV))
ALLOCATE (Q(LON,LAT,NIV))
ALLOCATE (XT(LON,LAT,NIV))
ALLOCATE (XP(LON,LAT,NIV))
ALLOCATE (XSDELTAT(LON,LAT,NIV))
ALLOCATE (ZLWCC(LON,LAT,NIV))
ALLOCATE (ZLWCI(LON,LAT,NIV))
ALLOCATE (ZKS1(LON,LAT,NIV))
ALLOCATE (ZKS2(LON,LAT,NIV))
ALLOCATE (FRAC(LON,LAT,NIV))
ALLOCATE (FRACSTOCK(LON,LAT,0:NIV))
ALLOCATE (RK(LON,LAT,NIV))
ALLOCATE (IIN(LON,LAT,NIV)) 
ALLOCATE (XPHFMM1(LON,LAT,NIV))   

#ifdef MASSy
ALLOCATE( NOx      (LON,LAT,NIV)) ; NOx   (:,:,:) = 0.0
ALLOCATE( HNO3     (LON,LAT,NIV)) ; HNO3  (:,:,:) = 0.0
ALLOCATE( N2O5     (LON,LAT,NIV)) ; N2O5  (:,:,:) = 0.0
ALLOCATE( HNO4     (LON,LAT,NIV)) ; HNO4  (:,:,:) = 0.0
ALLOCATE( HONO     (LON,LAT,NIV)) ; HONO  (:,:,:) = 0.0
ALLOCATE( ONIT     (LON,LAT,NIV)) ; ONIT  (:,:,:) = 0.0
ALLOCATE( PAN      (LON,LAT,NIV)) ; PAN   (:,:,:) = 0.0
#ifdef RACMOBUS
ALLOCATE( TPAN     (LON,LAT,NIV)) ; TPAN  (:,:,:) = 0.0
ALLOCATE( OLND     (LON,LAT,NIV)) ; OLND  (:,:,:) = 0.0
#endif
ALLOCATE( OLNN     (LON,LAT,NIV)) ; OLNN  (:,:,:) = 0.0
ALLOCATE( CLONO2   (LON,LAT,NIV)) ; CLONO2(:,:,:) = 0.0
ALLOCATE( CLNO2    (LON,LAT,NIV)) ; CLNO2 (:,:,:) = 0.0
ALLOCATE( BRONO2   (LON,LAT,NIV)) ; BRONO2(:,:,:) = 0.0
ALLOCATE( CLOx     (LON,LAT,NIV)) ; CLOx  (:,:,:) = 0.0
ALLOCATE( OCLO     (LON,LAT,NIV)) ; OCLO  (:,:,:) = 0.0
ALLOCATE( HCL      (LON,LAT,NIV)) ; HCL   (:,:,:) = 0.0
ALLOCATE( HOCL     (LON,LAT,NIV)) ; HOCL  (:,:,:) = 0.0
ALLOCATE( CL2      (LON,LAT,NIV)) ; CL2   (:,:,:) = 0.0
ALLOCATE( BRCL     (LON,LAT,NIV)) ; BRCL  (:,:,:) = 0.0
ALLOCATE( BROx     (LON,LAT,NIV)) ; BROx  (:,:,:) = 0.0
ALLOCATE( HBR      (LON,LAT,NIV)) ; HBR   (:,:,:) = 0.0
ALLOCATE( HOBR     (LON,LAT,NIV)) ; HOBR  (:,:,:) = 0.0
#endif
!        
!
!!!!!!!!!!!!!!!!!!!!!!!
!     Initialization  !
!!!!!!!!!!!!!!!!!!!!!!!
!
ZKHI_1=(0.2/1.0)*sqrt(18./34.)
!
TRAC(:,:,:)=0.
!
DO I=1,LON*LAT
  RPREC(I,1,0)=0.
  FRACSTOCK(I,1,0)=1.E-3
ENDDO
! 
DO I=1,LON*LAT
  DELTAP(I,1,1)=XPHFM(I,1,1)
ENDDO
!
!     Initialize RPREC (in m3 water/m2 surface/s)

DO I=1,LON*LAT*NIV
  RPREC(I,1,1)=1.E-3*(LSTPRECFLX(I,1,1)+SSTPRECFLX(I,1,1))
  FRACSTOCK(I,1,1)=1.E-3
  IIN(I,1,1)=0
ENDDO 
! 
!    Look for Lbot and Ltop
!
DO I=1,LON*LAT
  LTOP=NIV
  DO L=NIV,1,-1
    IF (RPREC(I,1,L) > SEUIL) THEN 
      LTOP=L
    ENDIF
  ENDDO
  LTOP=MAX(LTOP,1) 
  IF (LTOP.NE.NIV) THEN    
    DO L=LTOP,NIV
      IF (RPREC(I,1,L) > SEUIL) THEN
        LBOT=L
      ENDIF
    ENDDO
!
!    Computations have to be made down to
!    the layer just beyond the cloud base
!    (for reevaporation)
!    
    LBOT=MIN(LBOT+1,NIV)
    DO L=LTOP,LBOT
      IIN(I,1,L)=1
    ENDDO
  ENDIF
ENDDO
!
DO L=2,NIV
  DO I=1,LON*LAT
    IF (IIN(I,1,L) == 1) THEN
      XPHFMM1(I,1,L)=XPHFM(I,1,L-1)
    ENDIF
   ENDDO
ENDDO     
!
DO L=1,NIV
  DO I=1,LON*LAT
    IF (IIN(I,1,L) == 1) THEN
      RPRECM1(I,1,L)=RPREC(I,1,L-1)
    ENDIF
  ENDDO
ENDDO
!
DO L=2,NIV
  DO I=1,LON*LAT
    IF (IIN(I,1,L) == 1) THEN
      DELTAP(I,1,L)=XPHFM(I,1,L)-XPHFMM1(I,1,L)
    ENDIF
  ENDDO
ENDDO

#ifdef MASSy
iHNO3  =0
iN2O5  =0
iCLONO2=0
iCLNO2 =0
iBRONO2=0
iHNO4  =0
iHONO  =0
iONIT  =0
iPAN   =0
iTPAN  =0
iOLNN  =0
iOLND  =0
iNOx   =0
iNOy   =0
iOCLO  =0
iHCL   =0
iHOCL  =0
iCL2   =0
iCLOx  =0
iCLy   =0
iBRCL  =0
iHBR   =0
iHOBR  =0
iBROx  =0
iBRy   =0
DO JC=1,NTRA
  IF (TRANAM(JC) == 'HNO_3'  ) iHNO3  =JC
  IF (TRANAM(JC) == 'N_2O_5' ) iN2O5  =JC
  IF (TRANAM(JC) == 'ClONO_2') iCLONO2=JC
  IF (TRANAM(JC) == 'ClNO_2' ) iCLNO2 =JC
  IF (TRANAM(JC) == 'BrONO_2') iBRONO2=JC
  IF (TRANAM(JC) == 'HNO_4'  ) iHNO4  =JC
  IF (TRANAM(JC) == 'HONO'   ) iHONO  =JC
  IF (TRANAM(JC) == 'ONIT'   ) iONIT  =JC
  IF (TRANAM(JC) == 'PAN'    ) iPAN   =JC
  IF (TRANAM(JC) == 'TPAN'   ) iTPAN  =JC
  IF (TRANAM(JC) == 'OLNN'   ) iOLNN  =JC
  IF (TRANAM(JC) == 'OLND'   ) iOLND  =JC
  IF (TRANAM(JC) == 'NO_x'   ) iNOx   =JC
  IF (TRANAM(JC) == 'NO_y'   ) iNOy   =JC
  IF (TRANAM(JC) == 'OClO'   ) iOCLO  =JC
  IF (TRANAM(JC) == 'HCl'    ) iHCL   =JC
  IF (TRANAM(JC) == 'HOCl'   ) iHOCL  =JC
  IF (TRANAM(JC) == 'Cl_2'   ) iCL2   =JC
  IF (TRANAM(JC) == 'ClO_x'  ) iCLOX  =JC
  IF (TRANAM(JC) == 'Cl_y'   ) iCLy   =JC
  IF (TRANAM(JC) == 'BrCl'   ) iBRCL  =JC
  IF (TRANAM(JC) == 'HBr'    ) iHBR   =JC
  IF (TRANAM(JC) == 'HOBr'   ) iHOBR  =JC
  IF (TRANAM(JC) == 'BrO_x'  ) iBROX  =JC
  IF (TRANAM(JC) == 'Br_y'   ) iBRy   =JC
ENDDO
!
IF (iNOy == 0) CALL MOCAGE_STOP
IF (iCLy == 0) CALL MOCAGE_STOP
IF (iBRy == 0) CALL MOCAGE_STOP
!
IF (iHNO3   /= 0) HNO3  (:,:,:)=QQQM(:,:,:,iHNO3  )
IF (iN2O5   /= 0) N2O5  (:,:,:)=QQQM(:,:,:,iN2O5  )
IF (iCLONO2 /= 0) CLONO2(:,:,:)=QQQM(:,:,:,iCLONO2)
IF (iCLNO2  /= 0) CLNO2 (:,:,:)=QQQM(:,:,:,iCLNO2 )
IF (iBRONO2 /= 0) BRONO2(:,:,:)=QQQM(:,:,:,iBRONO2)
IF (iHNO4   /= 0) HNO4  (:,:,:)=QQQM(:,:,:,iHNO4  )
IF (iHONO   /= 0) HONO  (:,:,:)=QQQM(:,:,:,iHONO  )
IF (iONIT   /= 0) ONIT  (:,:,:)=QQQM(:,:,:,iONIT  )
IF (iPAN    /= 0) PAN   (:,:,:)=QQQM(:,:,:,iPAN   )
#ifdef RACMOBUS
IF (iTPAN   /= 0) TPAN  (:,:,:)=QQQM(:,:,:,iTPAN  )
IF (iOLND   /= 0) OLND  (:,:,:)=QQQM(:,:,:,iOLND  )
#endif
IF (iOLNN   /= 0) OLNN  (:,:,:)=QQQM(:,:,:,iOLNN  )
IF (iNOx    /= 0) NOx   (:,:,:)=QQQM(:,:,:,iNOx   )
IF (iOCLO   /= 0) OCLO  (:,:,:)=QQQM(:,:,:,iOCLO  )
IF (iHCL    /= 0) HCL   (:,:,:)=QQQM(:,:,:,iHCL   )
IF (iHOCL   /= 0) HOCL  (:,:,:)=QQQM(:,:,:,iHOCL  )
IF (iCL2    /= 0) CL2   (:,:,:)=QQQM(:,:,:,iCL2   )
IF (iCLOx   /= 0) CLOx  (:,:,:)=QQQM(:,:,:,iCLOx  )
IF (iBRCL   /= 0) BRCL  (:,:,:)=QQQM(:,:,:,iBRCL  )
IF (iHBR    /= 0) HBR   (:,:,:)=QQQM(:,:,:,iHBR   )
IF (iHOBR   /= 0) HOBR  (:,:,:)=QQQM(:,:,:,iHOBR  )
IF (iBROx   /= 0) BROx  (:,:,:)=QQQM(:,:,:,iBROx  )
#endif 
!
!     Compute Q (in m3/m3/s)
!     the precipitation generation rate in the grid box
!     IF Q > 0, precipitations are generated and rainout occurs
!     IF not, washout occurs in warm clouds and reevaporation occurs
!
!$omp parallel default(none) ,                                &
!$omp private (I,L,RHO),                                      &
!$omp shared(LON,LAT,NIV,IIN,XT,XP,DELTAZ,DELTAP,RA,GG),      &
!$omp shared(TGRIM,XPAM,QLIQ,QICE,RPREC,RPRECM1),             &
!$omp shared(ZLWCC,ZLWCI,XSDELTAT,ZKS1,ZKS2,Q,FRAC,RK,RNAIR), &
!$omp shared(SURF,MAIR)
!
!$omp do
DO I=1,LON*LAT*NIV
  IF (IIN(I,1,1) == 1) THEN
    XT(I,1,1)=TGRIM(I,1,1)
    XP(I,1,1)=XPAM(I,1,1)
    RHO=XP(I,1,1)/RA/XT(I,1,1)
    DELTAZ(I,1,1)=DELTAP(I,1,1)*RA*XT(I,1,1)/XP(I,1,1)/GG
    ZLWCC(I,1,1)=QLIQ(I,1,1)*RHO/1000.
    ZLWCI(I,1,1)=QICE(I,1,1)*RHO/910.
    XSDELTAT(I,1,1)=1./XT(I,1,1)-XT0_INV
    ZKS1(I,1,1)=1.7E-2 * EXP(-2090.*XSDELTAT(I,1,1))
    ZKS2(I,1,1)=6.3E-8 * EXP(-1495.*XSDELTAT(I,1,1))  
    Q(I,1,1)=(RPREC(I,1,1)-RPRECM1(I,1,1))/DELTAZ(I,1,1)
    FRAC(I,1,1)=1.E-3
    RK(I,1,1)=RKMIN
  ENDIF
ENDDO
!$omp enddo
!
!     Compute RNAIR, number of air moles in the grid box
!     RNAIR=S*DELTAP/G/MAIR
!      
!$omp do
DO L=1,NIV
  DO I=1,LON*LAT
    IF (IIN(I,1,L) == 1) THEN
      RNAIR(I,1,L)=SURF(I,1)*DELTAP(I,1,L)/GG/MAIR
    ENDIF
   ENDDO
ENDDO
!$omp enddo
!$omp end parallel
!
!     Fraction of the gridbox experiencing precipitations
!     
DO L=1,NIV
  DO I=1,LON*LAT
    IF (IIN(I,1,L) == 1) THEN
      IF (Q(I,1,L).GT.EPSILON) THEN  
        FRAC(I,1,L)=(Q(I,1,L)/(Q(I,1,L)+RKMIN*RLPW))
        FRAC(I,1,L)=MAX(FRAC(I,1,L),FRACSTOCK(I,1,L-1))
        FRACSTOCK(I,1,L)=FRAC(I,1,L)
!
!    Conversion rate K
!  
	RK(I,1,L)=RKMIN+(Q(I,1,L)/RLPW)
      ELSE
        FRACSTOCK(I,1,L)=FRACSTOCK(I,1,L-1)  
      ENDIF
    ENDIF
  ENDDO  
ENDDO
!
!    Computation of retention efficiency RIM
!    
DO JC=1,NTRA 
  IF ( (TRANAM(JC) == 'OP1').OR.              &
       (TRANAM(JC) == 'CH3OO    '))     THEN
    RIM(JC)=0.02
  ELSEIF ( (TRANAM(JC) == 'HCHO').OR.          &
          (TRANAM(JC) == 'CH_2O     ')) THEN
    RIM(JC)=0.02
  ELSEIF (  TRANAM(JC) == 'H_2O_2    ') THEN
    RIM(JC)=0.05
  ELSEIF (  TRANAM(JC) == 'HNO_3     ') THEN
    RIM(JC)=1.
  ELSE
    RIM(JC)=1. 
  ENDIF
ENDDO 
!
!
!
!$omp parallel default(none) ,                                          &
!$omp private (ALPHA,BETA,ZKH) ,                                        &                    
!$omp private (I,L,JC,ZKHI,ZKHC,RKJC,RLAMBDA,RKSEC) ,                   &
!$omp private (DELTANJC,DENUM) ,                                        &
!$omp private (QQM1,QQM2,QQM,COEF1,COEF2,RPRECMOY,RWC,RNJC),            &
!$omp shared (LON,LAT,NIV,NTRA,IOPT,ZKHI_1),                            &
!$omp shared (TRANAM,IIN,ZKS1,ZKS2,XSDELTAT,XKHENRY,ZLWCC,ZLWCI),       &
!$omp shared (Q,RIM,RK,FRAC,TRAC,QQQM,RNAIR,FRACSTOCK,RPRECM1,DELTAZ) , &
!$omp shared (RPREC,XT,XP)
!
ALLOCATE (ALPHA(LON,LAT,NIV)) 
ALLOCATE (BETA (LON,LAT,NIV))
ALLOCATE (ZKH(LON,LAT,NIV))
!
!$omp do
DO JC=1,NTRA    
!  
!
  IF (  TRANAM(JC) == 'N_2O_5   ') THEN
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN	
        ZKH(I,1,1)=2.*EXP(3400.*(XSDELTAT(I,1,1)))*1.E20
      ENDIF
    ENDDO  
  ELSEIF ( (TRANAM(JC) == 'HONO     ')  &
       .OR.(TRANAM(JC) == 'HNO2     ')) THEN
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN     
        ZKH(I,1,1)=5.00E+1*EXP(4900.*(XSDELTAT(I,1,1)))* &
                   5.10E-4*EXP(1250.*(XSDELTAT(I,1,1)))/ZHP
      ENDIF
    ENDDO
  ELSEIF (  TRANAM(JC) == 'HNO_3    ') THEN    
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN   	     
        ZKH(I,1,1)=2.10E+5*EXP(8700.*(XSDELTAT(I,1,1)))*15.4/ZHP	    
      ENDIF
    ENDDO
  ELSEIF (  TRANAM(JC) == 'SO_2     ') THEN    
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN 
        ZKH(I,1,1)=1.4*EXP(2900.*(XSDELTAT(I,1,1))) * &
                  (1.+(ZKS1(I,1,1)/ZHP)+(ZKS1(I,1,1)*ZKS2(I,1,1)/ZHP/ZHP))
      ENDIF
    ENDDO
  ELSEIF (  TRANAM(JC) == 'HO_2     ') THEN	     	    
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN
        ZKH(I,1,1)=4.E+3*EXP(5900.*(XSDELTAT(I,1,1)))*2.5E-5/ZHP
      ENDIF
    ENDDO
  ELSEIF (  TRANAM(JC) == 'ORA2     ') THEN 
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN
        ZKH(I,1,1)=8.80E+3*EXP(6400.*XSDELTAT(I,1,1))* &
                   1.70E-5*EXP(50.*XSDELTAT(I,1,1))/ZHP
      ENDIF
    ENDDO     
  ELSE
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN
        ZKH(I,1,1)= XKHENRY(JC,1)*EXP(-XKHENRY(JC,2)*XSDELTAT(I,1,1))                  
      ENDIF
    ENDDO
  ENDIF     
!        
!        
  IF ( (TRANAM(JC) == 'HNO_3') .OR. &
       (TRANAM(JC) == 'PB_210') )  THEN
    ZKHI=1.E9
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN	
        ZKHC=ZKH(I,1,1)*ZLWCC(I,1,1)*0.0825*XT(I,1,1)
        ALPHA(I,1,1)=ZKHC/(1.+ZKHC+ZKHI)
        BETA (I,1,1)=ZKHI/(1.+ZKHC+ZKHI)
      ENDIF
    ENDDO  
  ELSEIF (  TRANAM(JC) == 'H_2O_2   ') THEN
    DO I=1,LON*LAT*NIV
      IF (IIN(I,1,1) == 1) THEN	   
        ZKHI=ZKHI_1*XP(I,1,1)/     &
            (611.0 * exp(6138.*(ZKHI_2-(1./XT(I,1,1))))) &
           * ZKHI_3 * ZLWCI(I,1,1)
        ZKHC=ZKH(I,1,1)*ZLWCC(I,1,1)*0.0825*XT(I,1,1)		    
        ALPHA(I,1,1)=ZKHC/(1.+ZKHC+ZKHI)
        BETA (I,1,1)=ZKHI/(1.+ZKHC+ZKHI)
      ENDIF
    ENDDO	
  ELSE
    ZKHI=0.    
      DO I=1,LON*LAT*NIV
        IF (IIN(I,1,1) == 1) THEN
        ZKHC=ZKH(I,1,1)*ZLWCC(I,1,1)*0.0825*XT(I,1,1)
        ALPHA(I,1,1)=ZKHC/(1.+ZKHC+ZKHI)
        BETA (I,1,1)=ZKHI/(1.+ZKHC+ZKHI)
      ENDIF
    ENDDO    
  ENDIF	       	
!
!    For HNO3 and aerosols, kinetic process
!     
  IF ( (TRANAM(JC) == 'HNO_3') .OR. &
       (TRANAM(JC) == 'PB_210') )  THEN
!
    DO L=1,NIV 
      DO I=1,LON*LAT      
        IF (IIN(I,1,L) == 1) THEN
!
!      IF Q > 0, precipitations are generated and rainout occurs
!      IF not, washout occurs in warm clouds and reevaporation occurs
! 
!      Rainout
!  
          IF (Q(I,1,L) > EPSILON) THEN
!
            IF (IOPT == 1) THEN
!
!       Computation of RLAMBDA, fraction of tracer removed by rainout 
!       in the gridbox
!                            
              RKJC=(RIM(JC)*ALPHA(I,1,L)+BETA(I,1,L))*RK(I,1,L)
              RLAMBDA=FRAC(I,1,L)*(1.-EXP(-1.*RKJC*DELTAT))
              DELTANJC=RLAMBDA*QQQM(I,1,L,JC)*RNAIR(I,1,L)
              TRAC(I,1,JC)=TRAC(I,1,JC)+DELTANJC
              QQQM(I,1,L,JC)=QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L)
            ENDIF
!
!     Washout in warm clouds and reevaporation
!          
          ELSEIF (Q(I,1,L) < -EPSILON) THEN        
!
!     Washout in warm clouds 
!     applied over the areal fraction fracstock 
!     of the lowest rainout overhead
!
            IF (XT(I,1,L) >= 268.) THEN
              RKSEC=RKPRIM*RPREC(I,1,L)/FRACSTOCK(I,1,L)            
              RLAMBDA=FRACSTOCK(I,1,L)*(1.-EXP(-1.*RKSEC*DELTAT))
              DELTANJC=RLAMBDA*QQQM(I,1,L,JC)*RNAIR(I,1,L)
              TRAC(I,1,JC)=TRAC(I,1,JC)+DELTANJC
              QQQM(I,1,L,JC)=QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L)
            ENDIF        
!
!      Reevaporation, with a fraction of tracer 
!      precipitating from above equal
!      to half the relative evaporation rate of water
!      EXCEPT if the precipitations stop before arriving at the ground:
!      there is TOTAL reevaporation in this last gridbox
!
!      Another hypothesis for reevaporation
!      rate is DeltaP/2/P ("relative evaporation rate")
!           
            DENUM=MAX(RPRECM1(I,1,L),SEUIL)
            DELTANJC=-0.5*Q(I,1,L)*DELTAZ(I,1,L)/DENUM*TRAC(I,1,JC)
            IF (RPREC(I,1,L) < SEUIL) THEN
              QQQM(I,1,L,JC)=QQQM(I,1,L,JC)+TRAC(I,1,JC)/RNAIR(I,1,L)
              TRAC(I,1,JC)=0.
            ELSE
              QQQM(I,1,L,JC)=QQQM(I,1,L,JC)+DELTANJC/RNAIR(I,1,L)
              TRAC(I,1,JC)=TRAC(I,1,JC)-DELTANJC
            ENDIF        
!                
          ELSE       ! (ABS(Q)< ESPILON)
!                
!     Third case: mix between washout and rainout, when 
!    the flux variations are very small   
!             
!    Computations of rainout case
!
            RKJC=(RIM(JC)*ALPHA(I,1,L)+BETA(I,1,L))*RK(I,1,L)
            RLAMBDA=FRAC(I,1,L)*(1.-EXP(-1.*RKJC*DELTAT))
            DELTANJC=RLAMBDA*QQQM(I,1,L,JC)*RNAIR(I,1,L)
            QQM1=QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L)
!
            IF (XT(I,1,L) >= 268.) THEN
              RKSEC=RKPRIM*RPREC(I,1,L)/FRACSTOCK(I,1,L)            
              RLAMBDA=FRACSTOCK(I,1,L)*(1.-EXP(-1.*RKSEC*DELTAT))
              DELTANJC=RLAMBDA*QQQM(I,1,L,JC)*RNAIR(I,1,L)
              QQM2=QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L)
            ENDIF
!
            DENUM=MAX(RPRECM1(I,1,L),SEUIL)
            DELTANJC=-0.5*Q(I,1,L)*DELTAZ(I,1,L)/DENUM*TRAC(I,1,JC)
!
            IF (RPREC(I,1,L) < SEUIL) THEN
              QQM2=QQQM(I,1,L,JC)+TRAC(I,1,JC)/RNAIR(I,1,L)
            ELSE
              QQM2=QQQM(I,1,L,JC)+DELTANJC/RNAIR(I,1,L)
            ENDIF
!
            COEF1=(1.+Q(I,1,L)/EPSILON) * 0.5
            COEF2=1.-COEF1
            QQM=COEF1*QQM1+COEF2*QQM2
            DELTANJC=(QQQM(I,1,L,JC)-QQM)*RNAIR(I,1,L)
            TRAC(I,1,JC)=MAX(0.,TRAC(I,1,JC)+DELTANJC)
            QQQM(I,1,L,JC)=MAX(1.E-30,QQM)
!
          ENDIF
        ENDIF
      ENDDO
    ENDDO 
!    
  ELSE        
!	 
    DO L=1,NIV
      DO I=1,LON*LAT
        IF (IIN(I,1,L) == 1) THEN                  
!  
          IF (Q(I,1,L) > EPSILON) THEN
!
            IF (IOPT == 1) THEN                                   
!
!   Computation of RLAMBDA, fraction of tracer removed by rainout 
!   in the gridbox
!                            
              RKJC=(RIM(JC)*ALPHA(I,1,L)+BETA(I,1,L))*RK(I,1,L)
              RLAMBDA=FRAC(I,1,L)*(1.-EXP(-1.*RKJC*DELTAT))
              DELTANJC=RLAMBDA*QQQM(I,1,L,JC)*RNAIR(I,1,L)
              TRAC(I,1,JC)=TRAC(I,1,JC)+DELTANJC
              QQQM(I,1,L,JC)=QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L)
!
            ENDIF                          
!
!    Washout in warm clouds and reevaporation
!          
          ELSEIF (Q(I,1,L) < -EPSILON) THEN
!
!    For other gases, a Henry s law equilibrium process
!    describes washout AND reevaporation
!            
            RPRECMOY=0.5*(RPREC(I,1,L)+RPRECM1(I,1,L))
!
!    rain water content (m3 water/m3 air) in the grid box
!          
            RWC=RPRECMOY*DELTAT/FRACSTOCK(I,1,L)/DELTAZ(I,1,L)
!
!    Fraction of tracer submitted to washout is what is 
!    in equilibrium with rainwater
!        
            ZKHC=ZKH(I,1,L)*RWC*0.0825*XT(I,1,L)     
            RLAMBDA=ZKHC/(1.+ZKHC)
!
!     RNJC is the tracer quantity submitted to scavenging out of the
!     gridbox
!
            RNJC=FRACSTOCK(I,1,L)*QQQM(I,1,L,JC)*RNAIR(I,1,L)+TRAC(I,1,JC)
            DELTANJC=RLAMBDA*RNJC
            TRAC(I,1,JC)=TRAC(I,1,JC)+DELTANJC
            QQQM(I,1,L,JC)=MAX(1.E-30,QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L))
!
          ELSE       ! (ABS(Q)< ESPILON)
!                
!    Third case: mix between washout and rainout, when 
!    the flux variations are very small   
!             
!     Computations of rainout case

             RKJC=(RIM(JC)*ALPHA(I,1,L)+BETA(I,1,L))*RK(I,1,L)
             RLAMBDA=FRAC(I,1,L)*(1.-EXP(-1.*RKJC*DELTAT))
             DELTANJC=RLAMBDA*QQQM(I,1,L,JC)*RNAIR(I,1,L)
             QQM1=QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L)
!                  
             RPRECMOY=0.5*(RPREC(I,1,L)+RPRECM1(I,1,L))
             RWC=RPRECMOY*DELTAT/FRACSTOCK(I,1,L)/DELTAZ(I,1,L)
             ZKHC=ZKH(I,1,L)*RWC*0.0825*XT(I,1,L)     
             RLAMBDA=ZKHC/(1.+ZKHC)
!
!    RNJC is the tracer quantity submitted to scavenging out of the
!    gridbox
!
             RNJC=FRACSTOCK(I,1,L)*QQQM(I,1,L,JC)*RNAIR(I,1,L)+TRAC(I,1,JC)
             DELTANJC=RLAMBDA*RNJC
             QQM2=MAX(1.E-30,QQQM(I,1,L,JC)-DELTANJC/RNAIR(I,1,L))
!
             COEF1=(1.+Q(I,1,L)/EPSILON) * 0.5
             COEF2=1.-COEF1
             QQM=COEF1*QQM1+COEF2*QQM2
             DELTANJC=(QQQM(I,1,L,JC)-QQM)*RNAIR(I,1,L)
             TRAC(I,1,JC)=MAX(0.,TRAC(I,1,JC)+DELTANJC)
             QQQM(I,1,L,JC)=MAX(1.E-30,QQM)
!
          ENDIF        
!
        ENDIF
      ENDDO
    ENDDO  
!
  ENDIF        
!
!      
ENDDO
!$omp end do
!
DEALLOCATE (ALPHA)
DEALLOCATE (BETA)
DEALLOCATE (ZKH)
!
!$omp end parallel
!
!
#ifdef MASSy
QQQM(:,:,:,iNOy)=MAX( 0. , QQQM(:,:,:,iNOy)            &
   +     MIN(0.,QQQM(:,:,:,iHNO3  )-HNO3  (:,:,:))     &
   + 2.0*MIN(0.,QQQM(:,:,:,iN2O5  )-N2O5  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iCLONO2)-CLONO2(:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iCLNO2 )-CLNO2 (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iBRONO2)-BRONO2(:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iHNO4  )-HNO4  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iHONO  )-HONO  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iONIT  )-ONIT  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iPAN   )-PAN   (:,:,:))     &
#ifdef RACMOBUS
   +     MIN(0.,QQQM(:,:,:,iTPAN  )-TPAN  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iOLND  )-OLND  (:,:,:))     &
#endif
   +     MIN(0.,QQQM(:,:,:,iOLNN  )-OLNN  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iNOx   )-NOx   (:,:,:)) )
!
QQQM(:,:,:,iCLy)=MAX( 0. , QQQM(:,:,:,iCLy)            &
   +     MIN(0.,QQQM(:,:,:,iOCLO  )-OCLO  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iHCL   )-HCL   (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iCLONO2)-CLONO2(:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iHOCL  )-HOCL  (:,:,:))     &
   + 2.0*MIN(0.,QQQM(:,:,:,iCL2   )-CL2   (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iCLNO2 )-CLNO2 (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iBRCL  )-BRCL  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iCLOx  )-CLOx  (:,:,:)) )
!
QQQM(:,:,:,iBRy)=MAX( 0. , QQQM(:,:,:,iBRy)            &
   +     MIN(0.,QQQM(:,:,:,iHBR   )-HBR   (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iBRONO2)-BRONO2(:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iHOBR  )-HOBR  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iBRCL  )-BRCL  (:,:,:))     &
   +     MIN(0.,QQQM(:,:,:,iBROx  )-BROx  (:,:,:)) )
#endif
!
DEALLOCATE (RPREC)
DEALLOCATE (RPRECM1)
DEALLOCATE (RIM)
DEALLOCATE (TRAC)
DEALLOCATE (VALDEB)
DEALLOCATE (DELTAP)
DEALLOCATE (DELTAZ)
DEALLOCATE (RNAIR)
DEALLOCATE (Q)
DEALLOCATE (XT)
DEALLOCATE (XP)
DEALLOCATE (XSDELTAT)
DEALLOCATE (ZLWCC)
DEALLOCATE (ZLWCI)
DEALLOCATE (ZKS1)
DEALLOCATE (ZKS2)
DEALLOCATE (FRAC)
DEALLOCATE (FRACSTOCK)
DEALLOCATE (RK)
DEALLOCATE (IIN)
DEALLOCATE (XPHFMM1)
#ifdef MASSy
DEALLOCATE (NOx)
DEALLOCATE (HNO3)
DEALLOCATE (N2O5)
DEALLOCATE (HNO4)
DEALLOCATE (HONO)
DEALLOCATE (ONIT)
DEALLOCATE (PAN)
#ifdef RACMOBUS
DEALLOCATE (TPAN)
DEALLOCATE (OLND)
#endif
DEALLOCATE (OLNN)
DEALLOCATE (CLONO2)
DEALLOCATE (CLNO2)
DEALLOCATE (BRONO2)
DEALLOCATE (CLOx)
DEALLOCATE (OCLO)
DEALLOCATE (HCL)
DEALLOCATE (HOCL)
DEALLOCATE (CL2)
DEALLOCATE (BRCL)
DEALLOCATE (BROx)
DEALLOCATE (HBR)
DEALLOCATE (HOBR)
#endif
!  
! 
RETURN
END
