      SUBROUTINE TCVCLOUD(NIV,
     +                 PZ,PPRESS,PT,PQ,PQAC,PQAM,
     +                 PQTUR,PAMU,PTC,PQC,PLC,PGP,
     +                 PAM,PEU,PDU,PED,PDD,
     +                 LDEB,LFIN) 
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Tiedtke/Louis)
C   Purpose   : Compute cloud properties
C   Called by : 
C
C      Purpose
C      -------
C      CLOUD calculates all properties associated with subgridscale 
C      cloud mixing, i.e. massflux in updraft and downdraft, entrainment
C      and detrainment rates per level, and cloud properties: temperature,
C      moisture and liquid water and precipitation rate on each level
C      boundary.
C
C      Interface
C      ---------
C
C      CLOUD(PZ,PPRESS,PT,PQ,PQAC,PQAM,
C    1                 PQTUR,PAMU,PTC,PQC,PLC,PGP,
C    2                 PAM,PEU,PDU,PED,PDD) 
C
C	Input: (ALL VARIABLES ARE DEFINED ON LEVEL BOUNDARIES)
C		PZ 	GEOPOTENTIAL HEIGHT (M)
C		PPRESS  PRESSURE 	(PA)
C	        PT	TEMPERATURE 	(K)
C		PQ      MOISTURE 	(KG WATER/KG AIR)
C		PQAC	DIV(Q V) 	(KG WATER/KG AIR S**-1)
C		PQAM	DIV(V) 		(S**-1)
C		PQTUR	FQ SURF-AIR	(KG WATER/M**2 S**-1)
C
C	Output: VARIABLES DEFINED ON LEVEL BOUNDARIES:
C		PAMU	MASSFLUX 		(KG M**-2 S**-1)
C		PTC	UPDRAFT TEMPERATURE 	(K)
C		PQC	UPDRAFT MOISTURE 	(KG WATER/KG AIR)
C		PLC	UPDRAFT LIQUID WATER 	(KG WATER/KG AIR)
C		PGP	UPDRAFT PRECIPITATION 	(KG WATER M**-2 S**-1)
C		VARIABLES DEFINED ON EACH MODEL LEVEL:
C		PAM	AIR MASS 			(KG M**-2)
C		PEU	ENTRAINMENT IN UPDRAFT 		(KG M**-2 S**-1)
C		PDU	DETRAINMENT IN UPDRAFT 		(SAME)
C		PED	ENTRAINMENT IN DOWNDRAFT 	(KG M**-2 S**-1)
C		PDD	DETRAINMENT IN DOWNDRAFT 	(SAME)
C		KLC	LOWEST LEVEL IN CLOUD
C		KLT	INTERFACE ABOVE WHICH BUOYANCY LOST AT
C                       (CLOUD TOP)
C
C	Method
C	------
C
C	Cloud properties are calculated according to Tiedtke scheme
C
C	Externals
C	---------
C
C	Needs functions TCVQSAT and TCVDQSATDT
C
C	REFERENCES
C	----------
C
C	Tiedtke, MON. WEA. REV., 117, 1779-1800, 1989.
C
CCC
C
	USE CSTES_MOD
C
	IMPLICIT NONE	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: NIV,LDEB,LFIN
      REAL (KIND=8) :: PQTUR
	REAL (KIND=8) , DIMENSION (0:NIV) :: PZ,PT,PQ,PPRESS,PQAC,PQAM
      REAL (KIND=8) , DIMENSION (NIV) 	 :: PAM,PEU,PDU,PED,PDD
	REAL (KIND=8) , DIMENSION (0:NIV) :: PAMU,PTC,PQC,PLC,PGP	
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: NIVM1,NIVM1M1
      INTEGER :: JPITERMAX
      INTEGER :: KLC,KLT
	REAL (KIND=8) :: ZFCK,ZAMUB,ZAMU,ZLC,ZQC,ZTC
      REAL (KIND=8) :: ZDQ1,ZDQ2,ZSV,ZSCV,ZGP
	REAL (KIND=8) :: ZEPSU,ZDELTU,ZBETA
      INTEGER :: JL,JITER
C     Minimal moisture difference at cloud base 
C     between cloud and environment
      REAL (KIND=8) :: PPDQCMIN
      PARAMETER (PPDQCMIN=1.0E-4)
C     Constants for turbulent entrainment and detrainment rates
C     Penetrative convection
      REAL (KIND=8) :: PPEPSU,PPDELTU
      PARAMETER (PPEPSU =1.0E-4,PPDELTU =1.0E-4)
C     Shallow convection
      REAL (KIND=8) :: PPEPSUS,PPDELTUS
      PARAMETER (PPEPSUS=3.0E-4,PPDELTUS=3.0E-4)
C     Constants for precipitation parameterisation 
      REAL (KIND=8) :: PPKMIN,PPKVAL
      PARAMETER (PPKMIN=1500.,PPKVAL=2.E-3)
C     Parameter BETA determines the overshoot of the detrainment plumes
C     above level of no buoyancy (BETA=0 : no overshoot)
      REAL (KIND=8) :: PPBETA,PPBETAS
C     Penetrative convection
      PARAMETER (PPBETA =0.0)      
C     Shallow convection
      PARAMETER (PPBETAS=0.3)
C     No of iterations for saturation calculation
      PARAMETER(JPITERMAX=5)
C     External functions
      REAL (KIND=8) :: TCVQSAT,TCVDQSATDT
      REAL (KIND=8) , ALLOCATABLE :: ZPGP(:)
C
C      
	ALLOCATE (ZPGP(NIV))
C	
      NIVM1=NIV-1
      NIVM1M1=NIV-2
C
C 1./ Default cloud properties on each leyer boundary

      DO 1 JL=0,NIV
      
        PAMU(JL)=0.
        PTC (JL)=PT(JL)
	PQC (JL)=PQ(JL)
	PLC (JL)=0.
	PGP (JL)=0.
	
   1  CONTINUE
C
C 2./ Air masses (kg/m**2) in each layer, default entrainment/detrainment
C     and precipitation rates
C
      DO 2 JL=1,NIV
      
	PAM(JL)=(PPRESS(JL)-PPRESS(JL-1))/GG
	PEU(JL)=0.
	PDU(JL)=0.
	PED(JL)=0.
	PDD(JL)=0.
	ZPGP(JL)=0.
	
   2  CONTINUE
C
C     Find condensation level by lifting an air parcel until supersaturation 
C     occurs. We start the ascent with moisture and temperature of the upper 
C     boundary of the surface layer
C
      ZTC=PT(LFIN-1)
      ZQC=PQ(LFIN-1)
C

      DO 3 JL=LFIN-2,LDEB+1,-1
C       Preliminary value of parcel temperature (dry adiabatic ascent)
	ZTC=ZTC - (PZ(JL)-PZ(JL+1))*GG/CP
C       Check for supersaturation
        IF(ZQC.LT.0.)ZQC=0.
        IF(ZQC.GT.TCVQSAT(ZTC,PPRESS(JL))) THEN
C
C         If supersaturation is detected we adjust moisture and temperature by
C         condensation and set liquid water content equal to the condensate

	  ZDQ2=0.
	  DO 30 JITER=1,JPITERMAX
	    ZDQ1=(ZQC-TCVQSAT(ZTC,PPRESS(JL)))
     +           /(1.0+XLL/CP*TCVDQSATDT(ZTC,PPRESS(JL)))
	    ZQC=ZQC-ZDQ1
	    ZTC=ZTC+ZDQ1*XLL/CP
	    ZDQ2=ZDQ2+ZDQ1
30 	  CONTINUE
          ZLC=ZDQ2
C         Check if parcel is buoyant:
C         virtual temperature of parcel
          ZSCV=ZTC*(1.+0.608*ZQC-ZLC)
C         virtual temperature of environment
	  ZSV=PT(JL)*(1.+0.608*PQ(JL))
	  IF(ZSCV.GE.ZSV) THEN
C           If parcel is still buoyant then we have detected the cloud base
	    KLC=JL
	    GOTO 200
	  ELSE
C           If parcel is not buoyant then there is no convective cloud
	    GOTO 100
	  ENDIF
	ENDIF
3      CONTINUE
C
C     No condensation level found
      GOTO 100
200   CONTINUE
C
C     if we arrive here a cloud base is detected:
C     KLC is cloud base level, ZTC is cloud temperature, ZQC moisture (at
C     saturation) and ZLC the liquid water content in the updraft at base level
C     Calculate large scale moisture convergence belowe cloud base
C     (use trapezoidal integration formula)
C
      ZFCK=
     +    ((PQ(KLC)*PQAM(KLC)-PQAC(KLC))*PAM(KLC)+
     +     (PQ(KLC)*PQAM(NIV)-PQAC(NIV))*PAM(NIV))
      DO 4 JL=KLC+1,LFIN-1
        ZFCK=ZFCK+(PQ(KLC)*PQAM(JL)-PQAC(JL))*(PAM(JL)+PAM(JL+1))
 4    CONTINUE
C
C     Check for shallow or penetrative convection, 
C     set proper parameter values
C
      IF (ZFCK.GT.0.) THEN
C       penetrative convection
        ZEPSU =PPEPSU
        ZDELTU=PPDELTU
        ZBETA =PPBETA
      ELSE
C       shallow convection
        ZEPSU =PPEPSUS
        ZDELTU=PPDELTUS
        ZBETA =PPBETAS
      ENDIF
C
      ZFCK=PQTUR + 0.5*ZFCK
C    
C     If no moisture convergence then no cloud present
C
      IF (ZFCK.LE.0.) THEN
        GOTO 100
      ELSE 
        CONTINUE
      ENDIF
C
C     Massflux at base of cloud
C
      ZAMUB=ZFCK/(ZQC-PQ(KLC))
C
C     Set updraft entrainment rates below cloud base prop. to layer air masses
C     Set updraft detrainment rates below cloud base to zero
C
      DO 5 JL=LFIN,KLC+1,-1
        PEU(JL)=ZAMUB*PAM(JL)*GG/(PPRESS(NIV)-PPRESS(KLC))
	PDU(JL)=0.
 5    CONTINUE
C
C    Now calculate parcel ascent within cloud updraft
C    cloud mass flux               ZAMU
C    cloud moisture                ZQC
C    cloud temperature             ZTC
C    cloud liquid water            ZLC
C
C    Initialisation
      ZAMU=ZAMUB
      DO 6 JL=KLC,LDEB,-1
C       Mass entrainment and detrainment 
        PEU(JL)=ZEPSU*ZAMU*(PZ(JL-1)-PZ(JL))
	PDU(JL)=ZDELTU*ZAMU*(PZ(JL-1)-PZ(JL))
C
C       Preliminary values of cloud temperature, moisture 
C       and cloud liquid water
        ZTC=ZTC
     +        -GG*(PZ(JL-1)-PZ(JL))/CP
     +        +ZEPSU*(PZ(JL-1)-PZ(JL))*(PT(JL)-ZTC)
C       
	ZQC=ZQC
     +        +ZEPSU*(PZ(JL-1)-PZ(JL))*(PQ(JL)-ZQC)
C  
        ZLC=ZLC
     +        +ZEPSU*(PZ(JL-1)-PZ(JL))*(-ZLC)
C
C       Adjust moisture and temperature by condensation 
	ZDQ2=0.
	DO 60 JITER=1,JPITERMAX
	  ZDQ1=(ZQC-TCVQSAT(ZTC,PPRESS(JL)))
     +           /(1.0+XLL/CP*TCVDQSATDT(ZTC,PPRESS(JL)))
	  ZQC=ZQC-ZDQ1
	  ZTC=ZTC+ZDQ1*XLL/CP
	  ZDQ2=ZDQ2+ZDQ1
60 	CONTINUE
C
C       Precipitation rate out of layer JL
        IF((PZ(JL)+PZ(JL-1))*0.5-PZ(KLC) .GT. PPKMIN) THEN
          ZGP=PAM(JL)*PPKVAL/ZAMU
	ELSE
	  ZGP=0.
	ENDIF
C
C       Adjust liquid cloud water in updraf (use implicit
C       scheme to prevent instability)
C
        ZGP=ZGP*ZLC/(1+ZGP)
C
	ZPGP(JL)=ZGP*ZAMU
C
        ZLC=ZLC-ZGP+ZDQ2
C
C       Check for buoyancy 
C       Virtual temperature in updraft at upper boundary of layer jl:
        ZSCV=ZTC*(1.0 + 0.608*ZQC - ZLC)
C
C       Virtual temperature outside of cloud
	ZSV=PT(JL-1)*(1.0 + 0.608*PQ(JL-1))
C
	IF (ZSCV.LE.ZSV) THEN
	  KLT=JL
	  GOTO 65
	ENDIF
C
C       Updraft massflux at upper layer boundary
	ZAMU=ZAMU+PEU(JL)-PDU(JL)
C
C       Store cloud properties on upper layer boundary
        PTC(JL-1)=ZTC
        PQC(JL-1)=ZQC
        PLC(JL-1)=ZLC
C
  6   CONTINUE
C
      KLT=LDEB+1
C
 65   CONTINUE
C
C     Set detrainment in two layers above cloud
C
      PDU(KLT-1)=ZBETA*ZAMU
      PEU(KLT-1)=0.
      PDU(KLT)=(1-ZBETA)*ZAMU
      PEU(KLT)=0.
C      
C     Add up rainrate on each of the layer boundaries
C
      DO 7 JL=LDEB,LFIN
        PGP(JL)=PGP(JL-1) + ZPGP(JL)
 7    CONTINUE
C
C     Determine net mass flux on each of the layer boundaries
C
      DO 8 JL=LFIN,LDEB,-1
        PAMU(JL-1)=PAMU(JL)+PEU(JL)-PDU(JL)+PED(JL)-PDD(JL)
 8    CONTINUE
C      
      RETURN
C
C     No cloud present, set cloud base and top to zero and return
C
 100  CONTINUE
C
	DEALLOCATE (ZPGP)
C
C
      RETURN
      END
