      SUBROUTINE TCVCONVMA(NIV,LDEB,AM,EU,DU,ED,DD,AK,G)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Tiedtke/Louis) 
C   Purpose   : Compute convection matrix 
C   Called by : 
C
C     Calculate convection matrix for one vertical column based on 
C     given entrainment, detrainment and vertical diffusion coefficients.
C
C INPUT
C	AM:	AIRMASS (KG) PER M**2 IN EACH LAYER
C       EU:	ENTRAINMENT IN UPDRAFT IN EACH LAYER   (KG M**-2 S**-1)
C	DU:	DETRAINMENT IN UPDRAFT IN EACH LAYER   (KG M**-2 S**-1)
C	ED:	ENTRAINMENT IN DOWNDRAFT IN EACH LAYER (KG M**-2 S**-1)
C	DD:	DETRAINMENT IN DOWNDRAFT IN EACH LAYER (KG M**-2 S**-1)
C	AK:	DIFFUSION COEFFICIENT ON LAYER BOUNDARIES
C		TIMES DENSITY/DISTANCE BETWEEN ADJACENT LAYER CENTERS
C		(KG M**-2 S**-1)
C	DT0:	TIMESTEP (S)
C
C OUTPUT
C	G:	CONVECTION MATRIX
C
C     Method
C     ------
C     VERTICAL COORDINATE IS ASSUMED TO GO FROM TOP TO BOTTOM,
C 
C     Externals
C     ---------
C     SUBROUTINES:  DLINRG (IMSL) 
C
C     FIXED CASE WITH DOMINANT ENTRAINMENT FLUX AND INCLUDE CHECKS ON 
C     INPUT DATA
C
	USE EXPE_MOD
C
	IMPLICIT NONE
C		
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NIV,LDEB
      REAL (KIND=8) , DIMENSION (LDEB:NIV) :: AM,DU,DD,EU,ED
      REAL (KIND=8) , DIMENSION (LDEB-1:NIV) :: AK
      REAL (KIND=8) , DIMENSION (LDEB:NIV,LDEB:NIV) :: G
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: K,J,L
      REAL (KIND=8) :: XI
	REAL (KIND=8) , ALLOCATABLE :: F(:,:),FU(:,:),FD(:,:)      
      REAL (KIND=8) , ALLOCATABLE :: AMU(:),AMD(:)
C	      
C
	ALLOCATE (F(LDEB-1:NIV,LDEB:NIV))  ; F (:,:) = 0.0
	ALLOCATE (FU(LDEB-1:NIV,LDEB:NIV)) ; FU(:,:) = 0.0
	ALLOCATE (FD(LDEB-1:NIV,LDEB:NIV)) ; FD(:,:) = 0.0
	ALLOCATE (AMU(LDEB-1:NIV)) ; AMU(:) = 0.0
	ALLOCATE (AMD(LDEB-1:NIV)) ; AMD(:) = 0.0
C
      DO 1 K=LDEB-1,NIV
        DO 10 J=LDEB,NIV
          F (K,J)=0.
          FU(K,J)=0.
          FD(K,J)=0.
  10    CONTINUE
  1   CONTINUE
  
      DO 11 K=LDEB,NIV
        DO 12 L=LDEB,NIV
         G(K,L)=0.
 12     CONTINUE
 11   CONTINUE
C
C                                       Updraft
C                                       AMU is positive upward
      AMU(NIV)=0.
      DO 22 K=NIV,LDEB+1,-1
        AMU(K-1)=AMU(K) + EU(K) - DU(K)
        IF(AMU(K-1).GT.0.) THEN
C                                       We limit xi from below at 0. in case
C                                       there are inconsistent en/detrainment
C                                       rates. the case xi>1 should not occur
C                                       since EU and DU are >=0.
          XI=MAX(0.,1.-DU(K)/(AMU(K) + EU(K)))
          DO 20 J=K+1,NIV
            FU(K-1,J)=FU(K,J)*XI
20        CONTINUE
          FU(K-1,K)=EU(K)/AM(K)*XI
        ELSE
C                                       Set massflux at upper boundary to
C                                       strictly 0.
          AMU(K-1)=0.
        ENDIF
 22     CONTINUE
C                                       Downdraft
C                                       AMD is negative downward
      AMD(LDEB-1)=0.
      DO 3 K=LDEB,NIV-1
        AMD(K)=AMD(K-1) - ED(K) + DD(K)
        IF(AMD(K).LT.0.) THEN
          XI=MAX(0.,1.+DD(K)/(AMD(K-1)-ED(K)))
          DO 30 J=1,K-1
            FD(K,J)=FD(K-1,J)*XI
30        CONTINUE
          FD(K,K)=-ED(K)/AM(K)*XI
        ELSE
          AMD(K)=0.
        ENDIF
3     CONTINUE
C
      DO 4 K=LDEB,NIV-1
        DO 40 J=LDEB,NIV
          F(K,J)=FU(K,J)  + FD(K,J)
  40    CONTINUE
C       Subsidence
        F(K,K)  =F(K,K)   -(AMU(K) + AMD(K))/AM(K)
C       Diffusion terms
        F(K,K)  =F(K,K)   - AK(K)/AM(K)
        F(K,K+1)=F(K,K+1) + AK(K)/AM(K+1)
 4    CONTINUE
C
C     Generate forward matrix
      DO 5 K=LDEB,NIV
      DO 5 J=LDEB,NIV
          G(K,J)=DTCONV*(F(K-1,J) - F(K,J))
 5    CONTINUE
C
      DO 6 K=LDEB,NIV
        G(K,K)=G(K,K) + 1.0
 6    CONTINUE
C
	DEALLOCATE (F)
	DEALLOCATE (FU)
	DEALLOCATE (FD)
	DEALLOCATE (AMU)
	DEALLOCATE (AMD)
C
C
      RETURN
      END
