      SUBROUTINE TCVLOUIS(NIV,LDEB,PZ,PT,PU,PV,PK)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Tiedtke/Louis)
C   Purpose   : Compute vertical diffusion coefficients
C   Called by :
C
C     Purpose
C     -------
C     LOUIS calculates vertical diffusion coefficient based on vertical
C     stability of atmosphere.
C
C     Interface
C     ---------
C
C     CALL LOUIS(PZ,PT,PU,PV,PK)
C
C     Input: (all variables are defined on level boundaries)
C		PZ	GEOPOTENTIAL HEIGHT 	(M)
C		PT	TEMPERATURE  		(K)
C		PU	U-WINDCOMPONENT 	(M/S)
C		PV	V-WINDCOMPONENT 	(M/S)
C
C     Output: variables are defined on level boundaries:
C		PK	VERTICAL DIFFUSION COEFFICIENT
C						(KG M**-2 S**-1)
C
C     Method
C     ------
C
C     Vertical diffusion coefficients are calculated according to the
C     parameterisation developed by LOUIS
C
C     Externals
C     ---------
C
C     NONE
C
C     References
C     ----------
C
C     LOUIS, J.-F., A PARAMETRIC MODEL OF VERTICAL EDDY FLUXES IN THE
C     ATMOSPHERE, BOUNDARY LAYER METEOROLOGY, 17, 187-202, 1979.
C     ECMWF FORECAST MODEL DESCRIPTION, CHAPTER ... 19..
C
CCC
C
	USE CSTES_MOD
C
	IMPLICIT NONE	
C	
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NIV,LDEB
	REAL (KIND=8) , DIMENSION(0:NIV) :: PZ,PT,PU,PV
      REAL (KIND=8) , DIMENSION(LDEB-1:NIV) :: PK
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: JL,NIVM1
	REAL (KIND=8) :: ZH,ZAL2,ZDZM,ZDV2,ZRI,ZFH
	REAL (KIND=8) :: PPDV2MIN,PPB,PPC,PPD,PPALAMH,PPAK
C     Minimal velocity difference squared (M**2/S**2)
      PARAMETER (PPDV2MIN=0.01)
C     Other parameters
      PARAMETER(PPB=5.,PPC=5.,PPD=5.,PPALAMH=438.18,PPAK=0.4)
C
      NIVM1=NIV-1
      DO 1 JL=LDEB,NIVM1
C
C       Height above ground (GEOPOT. HEIGHT MIGHT BE NEGATIVE!)
        ZH=PZ(JL)-PZ(NIV)
C
C       Mixing length squared
        ZAL2=(PPAK*ZH/(1.+PPAK*ZH/PPALAMH))**2
C
C       Height difference between centers of adjacent layers
        ZDZM=(PZ(JL-1)-PZ(JL+1))/2.
C
C       Squared velocity difference between adjacent layers
        ZDV2=AMAX1(PPDV2MIN,((PU(JL-1)-PU(JL+1))**2 +
     .                       (PV(JL-1)-PV(JL+1))**2)/4.)
C     
C       Richardson number
        ZRI=GG*ZDZM*(CP*(PT(JL-1)-PT(JL+1))/2.+GG*ZDZM)/
     .         (CP*PT(JL)*ZDV2)
C
C       Stability functions:
        IF(ZRI.LT.0.) THEN
          ZFH=1.0 - 3.0*PPB*ZRI/
     .          (1.0 + 3.0*PPB*PPC*ZAL2*
     .        SQRT( -ZRI/ZH*
     .              (((1.+ZDZM/ZH)**(1./3.)-1.)/ZDZM)**3))
        ELSE
          ZFH=1./(1.0 + 3.0*PPB*ZRI*SQRT(1.0 + PPD*ZRI))
        ENDIF
C
C       Diffusiion coefficient
        PK(JL)=ZAL2*ZFH*SQRT(ZDV2)/ZDZM
C
 1    CONTINUE
C

      PK(LDEB-1)=0.0
      PK(NIV   )=0.0
C
      RETURN
      END

